@@ -98,7 +98,11 @@ task_summary = api.model('TaskSummary',
 
 
 task_list_get = api.model('TaskListGet', {
-    'tasks': fields.List(fields.Nested(task_summary))
+    'tasks': fields.List(fields.Nested(task_summary)),
+    'total_tasks': fields.Integer,
+    'limit': fields.Integer,
+    'offset': fields.Integer,
+
 })
 
 task_get = api.model('TaskGet', {
@@ -188,68 +192,26 @@ class TaskListAPI(Resource):
     @api.response(404, 'Could not find user or template')
     def get(self):
         args = self.get_request_parser.parse_args()
-        user = args['user']
-        project = args['project']
-        status = args['status']
-        template = args['template']
-        tags = args['tag']
-        application_name = args['application_name']
         offset = args['offset']
         limit = args['limit']
 
-        # Allow retrieval of tasks by user or in general
-        query = models.Task.query
+        tasks, total_nr_of_tasks = control.query_tasks(
+            user=args['user'],
+            project=args['project'],
+            status=args['status'],
+            template=args['template'],
+            tags=args['tag'],
+            application_name=args['application_name'],
+            offset=offset,
+            limit=limit,
+        )
 
-        # If not task_real_all, only allow own tasks
-        if not current_user.has_permission('task_read_all'):
-            user = current_user
-        # If tasks task_read_all allow to specify a user
-        elif user is not None:
-            user = get_object_from_arg(user,
-                                       models.User,
-                                       models.User.username,
-                                       skip_id=True,
-                                       allow_none=False)
-        # Default to current user if no user was specified
-        else:
-            user = current_user
-
-        # Query both tasks via user and via group
-        tasks_via_user_query = query.filter(models.Task.users.contains(user))
-        tasks_via_group_query = query.filter(models.Task.users_via_group.contains(user))
-        query = tasks_via_user_query.union(tasks_via_group_query)
-
-        if template is not None:
-            template = get_object_from_arg(args['template'],
-                                           models.TaskTemplate,
-                                           models.TaskTemplate.label,
-                                           allow_none=False)
-            query = query.filter(models.Task.template == template)
-
-        if project is not None:
-            query = query.filter(models.Task.project == project)
-
-        if status is not None:
-            query = query.filter_by(_status=status)
-        
-        if tags is not None:
-            for tag in tags:
-                tag_object = models.Tag.query.filter_by(name = tag).one_or_none()
-                if tag_object:
-                    query = query.filter(models.Task.tags.contains(tag_object))
-
-        if application_name is not None:
-            query = query.filter(models.Task.application_name == application_name)
-
-        if offset:
-            query = query.offset(offset)
-
-        if limit:
-            query = query.limit(limit)
-
-        tasks = query.all()
-
-        return {'tasks': tasks}
+        return {
+            'tasks': tasks,
+            'offset': offset,
+            'limit': limit,
+            'count': total_nr_of_tasks,
+        }
 
     @http_auth_required
     @permissions_required('task_add')
@@ -591,12 +553,17 @@ class TaskGroupListAPI(Resource):
         distribute_method = args['distribute_method']
         tasks = args['tasks']
 
-        taskgroup = control.insert_taskgroup(label=label,
-                                             callback_url=callback_url,
-                                             callback_content=callback_content,
-                                             distribute_in_group=distribute_in_group,
-                                             distribute_method=distribute_method,
-                                             tasks=tasks)
+        try:
+            taskgroup = control.insert_taskgroup(label=label,
+                                                 callback_url=callback_url,
+                                                 callback_content=callback_content,
+                                                 distribute_in_group=distribute_in_group,
+                                                 distribute_method=distribute_method,
+                                                 tasks=tasks)
+        except exceptions.TaskManagerError:
+            db.session.rollback()
+            raise
+
         # Commit and return
         db.session.commit()
         db.session.refresh(taskgroup)
